/*
	Author:		Nathaniel Meyer
	E-Mail:		nate@nutty.ca
	Website:	http://www.nutty.ca

	Desc:		Fragment program for calculating and drawing
				the mandelbrot on a shape's surface.
*/


// Textures
// tex0 is used to render to texture by the CPU
// if shaders are disabled. tex1 identifies the
// gradient texture used to colour the mandelbrot.
uniform sampler2D tex0;
uniform sampler2D tex1;

// Variables to be provided by application
// maxIteration		- A finite "infinity" number to prevent the
//					calculation loop from going on forever.
// real				- Starting x-coordinate in mandelbrot.
// imaginary		- Starting y-coordinate in mandelbrot.
// scaleX			- Maintain aspect ratio: 3.0 / window_width.
// scaleY			- Maintain aspect ratio: 2.0 / window_height.
uniform int maxIteration;
uniform float real;
uniform float imaginary;
uniform float scaleX;
uniform float scaleY;

// Keep track of complex number system.
float z;
float cfX;
float cfY;
float fX;
float fY;


/*
	Calculate mandelbrot
*/
int mandelbrot (int numIteration)
{
	int iteration = 0;
	float temp;

	// Zn+1 = Zn^2 + c
	while ( ((z = ((cfX*cfX + cfY*cfY))) <= (4.0)) && (iteration < numIteration) )
	{
		temp = cfX*cfX - cfY*cfY + fX;
		cfY = 2.0*cfX*cfY + fY;
		cfX = temp;

		++iteration;
	}

	return iteration;
}

/*
	Fragment Program Entry Point
*/
void main ()
{
	// Start location (center mandelbrot)
	fX = real + ((gl_TexCoord[0].s - 0.5) * scaleX);
	fY = imaginary + ((gl_TexCoord[0].t - 0.5) * scaleY);

	// Calculate mandelbrot
	int iteration = mandelbrot(maxIteration);

	// Process result
	if ( iteration == maxIteration )
	{
		// Not part of the mandelbrot solution
		gl_FragColor.xyz = 0.0.xxx;
	}
	else
	{
		float intensity;

		// Faster, but subject to colour banding
		//intensity = float(iteration) / float(maxIteration);

		// Normalized Iteration Count.
		// Slower, but reduces colour banding
		// Smoothing. Add a few extra counts to reduce error
		iteration += mandelbrot(1);
		z = (cfX*cfX + cfY*cfY);
		intensity = (float(iteration) - (log(log(sqrt(z))) / log(2.0))) / float(maxIteration);

		// If a texture gradient is not provided, this formula
		// will render a standard greyscale mandelbrot.
		//gl_FragColor.xyz = intensity;

		// Provide a 1D gradient for the mandelbrot to render
		// colours. It should be of sufficient width (1024 pixels)
		// to render smooth gradients.
		gl_FragColor.xyz = texture2D(tex1, vec2(intensity, 0)).xyz * 2.0;
	}

	// Set final output
	gl_FragColor.w = 1.0;
}